@@ -11,20 +11,20 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id$ *)
+(* $Id: zlib.mli 32 2016-06-10 09:30:19Z xleroy $ *)
 
 exception Error of string * string
 
 val compress:
   ?level: int -> ?header: bool -> 
-  (string -> int) -> (string -> int -> unit) -> unit
+  (bytes -> int) -> (bytes -> int -> unit) -> unit
 
 val compress_direct:
-  ?level: int -> ?header: bool -> (string -> int -> unit) ->
-  (string -> int -> int -> unit) * (unit -> unit)
+  ?level: int -> ?header: bool -> (bytes -> int -> unit) ->
+  (bytes -> int -> int -> unit) * (unit -> unit)
 
 val uncompress:
-  ?header: bool -> (string -> int) -> (string -> int -> unit) -> unit
+  ?header: bool -> (bytes -> int) -> (bytes -> int -> unit) -> unit
 
 type stream
 
@@ -36,17 +36,19 @@ type flush_command =
 
 external deflate_init: int -> bool -> stream = "camlzip_deflateInit"
 external deflate:
-  stream -> string -> int -> int -> string -> int -> int -> flush_command
+  stream -> bytes -> int -> int -> bytes -> int -> int -> flush_command
          -> bool * int * int
   = "camlzip_deflate_bytecode" "camlzip_deflate"
 external deflate_end: stream -> unit = "camlzip_deflateEnd"
 
 external inflate_init: bool -> stream = "camlzip_inflateInit"
 external inflate:
-  stream -> string -> int -> int -> string -> int -> int -> flush_command
+  stream -> bytes -> int -> int -> bytes -> int -> int -> flush_command
          -> bool * int * int
   = "camlzip_inflate_bytecode" "camlzip_inflate"
 external inflate_end: stream -> unit = "camlzip_inflateEnd"
 
-external update_crc: int32 -> string -> int -> int -> int32
+external update_crc: int32 -> bytes -> int -> int -> int32
+                   = "camlzip_update_crc32"
+external update_crc_string: int32 -> string -> int -> int -> int32
                    = "camlzip_update_crc32"
